import NormalMessage from "@root/Message/variants/NormalMessage";
import React, { memo } from "react";
import MessageContainer from "@root/Message/MessageContainer";
import { MessageType } from "discord-api-types/v10";
import GuildMemberJoin from "@root/Message/variants/GuildMemberJoin";
import GuildDiscoveryRequalified from "@root/Message/variants/GuildDiscoveryRequalified";
import GuildDiscoveryGracePeriodInitialWarning from "@root/Message/variants/GuildDiscoveryGracePeriodInitialWarning";
import GuildDiscoveryGracePeriodFinalWarning from "@root/Message/variants/GuildDiscoveryGracePeriodFinalWarning";
import GuildDiscoveryDisqualified from "@root/Message/variants/GuildDiscoveryDisqualified";
import ChannelFollowAdd from "@root/Message/variants/ChannelFollowAdd";
import ChannelNameChange from "@root/Message/variants/ChannelNameChange";
import GuildBoost from "@root/Message/variants/GuildBoost";
import BoostTierUpgrade from "@root/Message/variants/BoostTierUpgrade";
import ChannelPinnedMessage from "@root/Message/variants/ChannelPinnedMessage";
import RecipientAdd from "@root/Message/variants/RecipientAdd";
import RecipientRemove from "@root/Message/variants/RecipientRemove";
import ThreadCreated from "@root/Message/variants/ThreadCreated";
import { useConfig } from "@root/core/ConfigContext";
function MessageTypeSwitch(props) {
    switch (props.message.type) {
        case MessageType.ChannelPinnedMessage:
            return (React.createElement(ChannelPinnedMessage, { createdAt: props.message.timestamp, author: props.message.author }));
        case MessageType.UserJoin:
            return (React.createElement(GuildMemberJoin, { createdAt: props.message.timestamp, author: props.message.author }));
        case MessageType.GuildDiscoveryRequalified:
            return React.createElement(GuildDiscoveryRequalified, { createdAt: props.message.timestamp });
        case MessageType.GuildBoostTier1:
        case MessageType.GuildBoostTier2:
        case MessageType.GuildBoostTier3:
            return (React.createElement(BoostTierUpgrade, { content: props.message.content, createdAt: props.message.timestamp, author: props.message.author, type: props.message.type }));
        case MessageType.GuildBoost:
            return (React.createElement(GuildBoost, { createdAt: props.message.timestamp, author: props.message.author, content: props.message.content }));
        case MessageType.RecipientAdd:
            return (React.createElement(RecipientAdd, { createdAt: props.message.timestamp, author: props.message.author, target: props.message.mentions[0] }));
        case MessageType.RecipientRemove:
            return (React.createElement(RecipientRemove, { createdAt: props.message.timestamp, author: props.message.author, target: props.message.mentions[0] }));
        case MessageType.ChannelNameChange:
            return (React.createElement(ChannelNameChange, { createdAt: props.message.timestamp, author: props.message.author, content: props.message.content }));
        case MessageType.ThreadCreated:
            return (React.createElement(ThreadCreated, { createdAt: props.message.timestamp, thread: props.message.thread, author: props.message.author, messageId: props.message.id, messageReference: props.message.message_reference, messageContent: props.message.content }));
        case MessageType.Reply:
        case MessageType.Default:
        case MessageType.ChatInputCommand:
            return React.createElement(NormalMessage, Object.assign({}, props));
        case MessageType.ChannelFollowAdd:
            return (React.createElement(ChannelFollowAdd, { createdAt: props.message.timestamp, author: props.message.author, content: props.message.content }));
        case MessageType.GuildDiscoveryGracePeriodInitialWarning:
            return (React.createElement(GuildDiscoveryGracePeriodInitialWarning, { createdAt: props.message.timestamp }));
        case MessageType.GuildDiscoveryGracePeriodFinalWarning:
            return (React.createElement(GuildDiscoveryGracePeriodFinalWarning, { createdAt: props.message.timestamp }));
        case MessageType.GuildDiscoveryDisqualified:
            return React.createElement(GuildDiscoveryDisqualified, { createdAt: props.message.timestamp });
        case MessageType.ContextMenuCommand:
            return React.createElement(NormalMessage, Object.assign({}, props, { isContextMenuInteraction: true }));
        case MessageType.ThreadStarterMessage:
            return (React.createElement(NormalMessage, Object.assign({}, props, { message: props.message.referenced_message, noThreadButton: true })));
        default: {
            // todo: lock behind a debug mode
            const errorMessage = Object.assign(Object.assign({}, props.message), { type: MessageType.Default, content: `Unknown message type \`${props.message.type}\`\n\n\`\`\`json\n${JSON.stringify(props.message, null, 2)}\n\`\`\`` });
            return (React.createElement(NormalMessage, { message: errorMessage, isFirstMessage: props.isFirstMessage, isHovered: props.isHovered }));
        }
    }
}
function Message(props) {
    var _a;
    const { messageButtons } = useConfig();
    const buttonOptions = (_a = messageButtons === null || messageButtons === void 0 ? void 0 : messageButtons(props.message)) !== null && _a !== void 0 ? _a : [];
    if (props.showButtons)
        return (React.createElement(MessageContainer, { buttons: buttonOptions },
            React.createElement(MessageTypeSwitch, Object.assign({}, props))));
    return React.createElement(MessageTypeSwitch, Object.assign({}, props));
}
export default memo(Message);
//# sourceMappingURL=index.js.map