import React, { memo, useMemo } from "react";
import MessageAuthor from "@root/Message/MessageAuthor";
import Content from "@root/Content";
import Moment from "moment";
import Tooltip from "@root/Tooltip";
import getAvatar from "@utils/getAvatar";
import LargeTimestamp from "@root/Message/LargeTimestamp";
import ChatTag from "@root/ChatTag";
import * as Styles from "@root/Message/style/message";
import { MessageType, } from "discord-api-types/v10";
function getMiniAvatarUrl(referencedMessage, interaction) {
    const getAvatarSettings = {
        size: 16,
        animated: false,
    };
    if (interaction !== undefined)
        return getAvatar(interaction.user, getAvatarSettings);
    if (referencedMessage !== undefined && referencedMessage !== null)
        return getAvatar(referencedMessage.author, getAvatarSettings);
    return null;
}
function getMiniUserName(referencedMessage, interaction) {
    var _a, _b;
    if (interaction !== undefined)
        return interaction.user.username;
    return (_b = (_a = referencedMessage === null || referencedMessage === void 0 ? void 0 : referencedMessage.author) === null || _a === void 0 ? void 0 : _a.username) !== null && _b !== void 0 ? _b : null;
}
function getDominantRoleColor(referencedMessage) {
    // todo: make work
    // if (referencedMessage !== null) {
    //   const roleIds = referencedMessage.author.roles ?? [];
    //   const [role] = roleIds
    //     .map((id) => generalStore.guild?.roles.find((r) => r.id === id))
    //     .filter((r) => r !== undefined && r.color !== 0)
    //     .sort((a, b) => b.position - a.position);
    //
    //   const colorHex = role?.color ?? null;
    //   return colorHex > 0 ? `#${colorHex.toString(16).padStart(6, "0")}` : "#fff";
    // }
    return null;
}
const ReplyInfo = memo((props) => {
    var _a;
    const miniAvatarUrl = useMemo(() => getMiniAvatarUrl(props.referencedMessage, props.interaction), [props.referencedMessage, props.interaction]);
    const miniUserName = useMemo(() => getMiniUserName(props.referencedMessage, props.interaction), [props.referencedMessage, props.interaction]);
    const miniUserNameColorHex = getDominantRoleColor(props.referencedMessage);
    const unknownReply = !props.referencedMessage && !props.interaction;
    return (React.createElement(Styles.ReplyInfo, null,
        React.createElement(Styles.ReplySpine, null),
        unknownReply ? (React.createElement(React.Fragment, null,
            React.createElement(Styles.UnknownReplyIcon, { width: 12, height: 12, svg: "IconUnknownReply" }),
            React.createElement(Styles.UnknownReply, null, "Original message was deleted or is unknown."))) : (React.createElement(Styles.ReplyUser, null,
            React.createElement(Styles.MiniUserAvatar, { src: miniAvatarUrl }),
            props.referencedMessage && (React.createElement(ChatTag, { author: props.referencedMessage.author, crosspost: !!(props.referencedMessage.flags & (1 << 1)), referenceGuild: (_a = props.referencedMessage.message_reference) === null || _a === void 0 ? void 0 : _a.guild_id })),
            React.createElement(Styles.MiniUserName, { style: { color: miniUserNameColorHex } },
                props.mentioned && "@",
                miniUserName))),
        props.referencedMessage ? (React.createElement(Content, { message: props.referencedMessage, isReplyContent: true })) : (props.interaction && (React.createElement(Styles.SlashCommand, null,
            "used",
            " ",
            React.createElement(Styles.SlashCommandText, null,
                !props.isContextMenuInteraction ? "/" : "",
                props.interaction.name))))));
});
function NormalMessage(props) {
    var _a, _b, _c;
    const shouldShowReply = props.message.type === MessageType.Reply ||
        Boolean(props.message.interaction);
    const isUserMentioned = useMemo(() => {
        var _a, _b;
        // todo: make work
        return (_b = (_a = props.overrides) === null || _a === void 0 ? void 0 : _a.userMentioned) !== null && _b !== void 0 ? _b : false;
        // const user = authStore.user;
        //
        // if (!user) return false;
        //
        // if (!("_id" in user)) return false;
        //
        // const userMentioned = props.message.mentions.find(
        //   (mention) => mention.id === user._id
        // );
        //
        // return Boolean(userMentioned);
    }, [props.message.mentions, (_a = props.overrides) === null || _a === void 0 ? void 0 : _a.userMentioned]);
    if (props.isFirstMessage)
        return (React.createElement(Styles.Message, { mentioned: isUserMentioned },
            shouldShowReply && (React.createElement(ReplyInfo, { referencedMessage: props.message.referenced_message, mentioned: props.message.mentions.some((m) => { var _a; return m.id === ((_a = props.message.referenced_message) === null || _a === void 0 ? void 0 : _a.author.id); }), interaction: props.message.interaction, isContextMenuInteraction: props.isContextMenuInteraction })),
            React.createElement(Styles.MessageHeaderBase, null,
                React.createElement(MessageAuthor, { author: props.message.author, avatarAnimated: (_b = props.isHovered) !== null && _b !== void 0 ? _b : false, crosspost: !!(props.message.flags & (1 << 1)), referenceGuild: (_c = props.message.message_reference) === null || _c === void 0 ? void 0 : _c.guild_id }),
                props.hideTimestamp || (React.createElement(LargeTimestamp, { timestamp: props.message.timestamp }))),
            React.createElement(Content, { message: props.message, noThreadButton: props.noThreadButton })));
    return (React.createElement(Styles.Message, { mentioned: isUserMentioned },
        React.createElement(Tooltip, { placement: "top", overlay: Moment(props.message.timestamp).format("LLLL"), mouseEnterDelay: 1 },
            React.createElement(Styles.SmallTimestamp, { dateTime: props.message.timestamp }, Moment(props.message.timestamp).format("h:mm A"))),
        React.createElement(Content, { message: props.message, noThreadButton: props.noThreadButton })));
}
export default NormalMessage;
//# sourceMappingURL=NormalMessage.js.map