var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import memoize from "memoizee";
import * as R from "ramda";
import baseRules from "@root/markdown/render/ast";
import { Highlighter } from "@root/markdown/render/elements";
import * as Styles from "./elements";
import { astToString, flattenAst, recurse } from "@root/markdown/render/util";
import SimpleMarkdown from "simple-markdown";
import TextSpoiler from "@root/markdown/render/elements/TextSpoiler";
import { Timestamp } from "./elements/Timestamp";
import React from "react";
import { InlineCode } from "@root/markdown/render/elements/code/style";
function parserFor(rules, returnAst) {
    const parser = SimpleMarkdown.parserFor(rules);
    const renderer = SimpleMarkdown.outputFor(rules, "react");
    return memoize((input = "", inline = true, state = {}, transform = null) => {
        if (!inline) {
            input += "\n\n";
        }
        const parse = R.pipe.apply(this, [parser, flattenAst, transform, !returnAst && renderer].filter(Boolean));
        return parse(input, Object.assign({ inline }, state));
    }, {
        normalizer: (...args) => JSON.stringify(args),
    });
}
function createRules(rule) {
    const { paragraph, url, link, codeBlock, inlineCode, blockQuote, spoiler, timestamp, command, } = rule;
    return Object.assign(Object.assign({}, rule), { s: {
            order: rule.u.order,
            match: SimpleMarkdown.inlineRegex(/^~~([\s\S]+?)~~(?!_)/),
            parse: rule.u.parse,
            react: (node, recurseOutput, state) => (React.createElement("s", { key: state.key }, recurseOutput(node.content, state))),
        }, paragraph: Object.assign(Object.assign({}, paragraph), { react: (node, recurseOutput, state) => (React.createElement("p", { key: state.key }, recurseOutput(node.content, state))) }), url: Object.assign(Object.assign({}, url), { match: SimpleMarkdown.inlineRegex(/^((https?|steam):\/\/[^\s<]+[^<.,:;"')\]\s])/) }), link: Object.assign(Object.assign({}, link), { react: (node, recurseOutput, state) => (React.createElement(Styles.Link, { title: node.title || astToString(node.content), href: SimpleMarkdown.sanitizeUrl(node.target), target: "_blank", rel: "noreferrer", key: state.key }, recurseOutput(node.content, state))) }), inlineCode: Object.assign(Object.assign({}, inlineCode), { react: (node, recurseOutput, state) => (React.createElement(InlineCode, { key: state.key }, recurse(node, recurseOutput, state))) }), codeBlock: Object.assign(Object.assign({}, codeBlock), { react: (node, recurseOutput, state) => (React.createElement(Highlighter, { key: state.key, language: node.lang }, recurse(node, recurseOutput, state))) }), blockQuote: Object.assign(Object.assign({}, blockQuote), { react: (node, recurseOutput, state) => (React.createElement(Styles.QuoteContainer, { key: state.key },
                React.createElement(Styles.QuoteBar, null),
                React.createElement(Styles.Quote, null, recurse(node, recurseOutput, state)))) }), spoiler: Object.assign(Object.assign({}, spoiler), { react: (node, recurseOutput, state) => (React.createElement(TextSpoiler, { content: recurse(node, recurseOutput, state) })) }), timestamp: Object.assign(Object.assign({}, timestamp), { react: (data) => React.createElement(Timestamp, { data: data }) }), command: Object.assign(Object.assign({}, command), { react: ({ name }) => "/" + name }) });
}
const rulesWithoutMaskedLinks = createRules(Object.assign(Object.assign({}, baseRules), { link: Object.assign(Object.assign({}, baseRules.link), { match: () => null }) }));
const parse = parserFor(rulesWithoutMaskedLinks);
export const parseAllowLinks = parserFor(createRules(baseRules));
export const parseEmbedTitle = parserFor(R.omit(["codeBlock", "br", "mention", "channel", "roleMention"], rulesWithoutMaskedLinks));
function Markdown({ children: content, mentions, users, }) {
    return content ? parse(content, undefined, { mentions, users }) : null;
}
(function (Markdown) {
    Markdown.withComponent = (Component) => (_a) => {
        var { children } = _a, props = __rest(_a, ["children"]);
        return (React.createElement(Component, Object.assign({}, props),
            React.createElement(Markdown, null, children)));
    };
})(Markdown || (Markdown = {}));
export function LinkMarkdown({ children: content, mentions, users, }) {
    return content
        ? parseAllowLinks(content, undefined, { mentions, users })
        : null;
}
(function (LinkMarkdown) {
    LinkMarkdown.withComponent = (Component) => (_a) => {
        var { children } = _a, props = __rest(_a, ["children"]);
        return (React.createElement(Component, Object.assign({}, props),
            React.createElement(LinkMarkdown, null, children)));
    };
})(LinkMarkdown || (LinkMarkdown = {}));
export default Markdown;
//# sourceMappingURL=index.js.map